#include "game.h"
#include "tiles.h"
#include "walls.h"
#include "player.h"
#include "../common/sources/system/fileCache.h"
#include "../common/sources/system/cgraphics2d.h"
#include <QDebug>
#include <QPainter>
#include "interface.h"
#include "bridge.h"

CGame   game;

//-----------------------------------------------------------------------------------------
CGame::CGame()
{
}

//-----------------------------------------------------------------------------------------
void CGame::drawWall(QImage* image, CTile* tile, CVec2 tablePos, EWallSide side, bool flip)
{
    EWallSide playerSide = player.getWallSide(side);

    if (tile->mWalls[playerSide].getType() != eWallNothing)
    {
        const char* fileName = walls.getFilename(tablePos, side, flip);

        if (fileName[0] != 0)
        {
            // draw the wall
            QImage  wallGfx = fileCache.getImage(fileName);
            CVec2 gfxPos = walls.getGfxPos(tablePos, side, flip);

            graph2D.drawImage(image, gfxPos, wallGfx, 0, flip);

            // draw the ornates
            CWall*  wall = &tile->mWalls[playerSide];

            if (wall->getType() == eWallSimple)
            {
                //------------------------------------------------------------------------------
                // simple wall
                int ornate = wall->getOrnateParam("Ornate");
                walls.drawOrnate(image, tablePos, side, ornate);
            }
            else if (wall->getType() == eWallPortrait)
            {
                //------------------------------------------------------------------------------
                // champion mirror
                walls.drawOrnate(image, tablePos, side, ORNATE_MIRROR);

                if (tablePos == CVec2(2, 3) && side == eWallSideUp)
                {
                    // if the mirror is right in front of us, we draw the champion's head
                    int champion = wall->getIntParam("Champion");
                    bool isEmpty = wall->getBoolParam("isEmpty");

                    if (isEmpty == false)
                    {
                        CRect   rect = interface.getChampionPortraitRect(champion);
                        QImage  portraits = fileCache.getImage("gfx/interface/ChampPortraits.png");
                        CVec2   pos(96, 68);
                        graph2D.drawImageAtlas(image, pos, portraits, rect);

                        int lastChampChosed;
                        for (lastChampChosed = 0; lastChampChosed < 4; ++lastChampChosed)
                            if (characters[lastChampChosed].portrait == -1)
                                break;

                        if (lastChampChosed != 4)
                        {
                            CRect   mouseRect(pos, CVec2(pos.x + CHAMP_PORTRAIT_WIDTH - 1, pos.y + CHAMP_PORTRAIT_HEIGHT - 1));
                            mouse.addArea(eMouseAreaG_Champion, mouseRect, (void*)wall);
                        }
                    }
                }
            }
        }
    }
}

//-----------------------------------------------------------------------------------------
void CGame::displayMainView(QImage* image)
{
    if (interface.isInInventory() == false)
    {
        // draw the floor and the ceiling
        QImage  floorGfx = fileCache.getImage("gfx/3DView/Floor.png");
        QImage  ceilingGfx = fileCache.getImage("gfx/3DView/Ceiling.png");

        bool flip = (player.pos.x + player.pos.y + player.dir) & 1;

        graph2D.drawImage(image, CVec2(0, 33), ceilingGfx, 0, flip);
        graph2D.drawImage(image, CVec2(0, 72), floorGfx, 0, flip);

        // draw the walls
        CVec2   tablePos;
        CVec2   localPos;
        CVec2   mapPos;

        for (int y = 0; y < WALL_TABLE_HEIGHT; ++y)
            for (int x = 0; x < WALL_TABLE_WIDTH; ++x)
            {
                tablePos = walls.getDrawSequence(CVec2(x, y));
                localPos = walls.getMapPos(tablePos);
                mapPos = player.getPosFromLocal(localPos);
                CTile*  tile = map.getTile(mapPos);

                if (tile != NULL)
                {
                    // draw the walls
                    drawWall(image, tile, tablePos, eWallSideUp, flip);
                    drawWall(image, tile, tablePos, eWallSideLeft, flip);
                    drawWall(image, tile, tablePos, eWallSideRight, flip);

                    // draw the floor ornates
                    if (tile->getType() == eTileGround)
                    {
                        //------------------------------------------------------------------------------
                        // ground tile
                        int ornate = tile->getOrnateParam("Ornate");
                        tiles.drawFloorOrnate(image, tablePos, ornate);
                    }

                    // draw objects
                    // draw ennemies
                }
            }
    }
}

//-----------------------------------------------------------------------------------------
QImage CGame::mainLoop()
{
    QImage image(SCREEN_WIDTH,
                 SCREEN_HEIGHT,
                 QImage::Format_RGBA8888);

    mouse.clearAreas();

    // clear the screen
    image.fill(QColor(0, 0, 0));

    // display
    displayMainView(&image);
    interface.display(&image);

    // processing
    mouse.update();
    SMouseArea* clickedArea = mouse.clickedArea();

    update(clickedArea);
    interface.update(clickedArea);

    return image;
}

//-----------------------------------------------------------------------------------------
void CGame::update(SMouseArea* clickedArea)
{
    if (clickedArea != NULL)
    {
        if (mouse.mButtonPressing == true)
        {
            if (clickedArea->type == eMouseAreaG_Champion)
            {
                lastMirrorClicked = (CWall*)clickedArea->param1;

                int lastChampChosed;
                for (lastChampChosed = 0; lastChampChosed < 4; ++lastChampChosed)
                    if (characters[lastChampChosed].portrait == -1)
                        break;

                characters[lastChampChosed].fromDB(lastMirrorClicked->getIntParam("Champion"));
                interface.enterResurrect(lastChampChosed);
            }
        }
    }
}
